//********************************************************************************************************************************
//*                                                                                                                              *
//*     CDDB-FreeDB Library V1.2 Tutorial program  3delite 2008-2011                                                            *
//*     See CDDB-FreeDB Library Readme.txt for details                                                                           *
//*                                                                                                                              *
//* Two licenses are available if you like and use this library:                                                                 *
//* Shareware License: 50 Euros                                                                                                  *
//* Commercial License: 250 Euros                                                                                                *
//*                                                                                                                              *
//*     http://www.3delite.hu/Object%20Pascal%20Developer%20Resources/cddbfreedblibrary.html                                     *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit Unit1;

interface

uses
    Windows,
    Messages,
    SysUtils,
    Variants,
    Classes,
    Graphics,
    Controls,
    Forms,
    Dialogs,
    CDDBLibraryDefs,
    StdCtrls,
    BASSCD,
    BASS,
    ComCtrls;

type
  TForm1 = class(TForm)
    ComboBox1: TComboBox;
    Label1: TLabel;
    Button1: TButton;
    Edit1: TEdit;
    Label2: TLabel;
    Button2: TButton;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Memo1: TMemo;
    EditPort: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Edit2: TEdit;
    Memo2: TMemo;
    GroupBox3: TGroupBox;
    Button3: TButton;
    Edit3: TEdit;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Edit4: TEdit;
    Edit5: TEdit;
    Label10: TLabel;
    Edit6: TEdit;
    Label11: TLabel;
    Label13: TLabel;
    Edit7: TEdit;
    Edit9: TEdit;
    Memo3: TMemo;
    Label14: TLabel;
    ComboBox3: TComboBox;
    Label15: TLabel;
    Label12: TLabel;
    Edit8: TEdit;
    Edit10: TEdit;
    Edit11: TEdit;
    Edit12: TEdit;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Edit13: TEdit;
    Edit14: TEdit;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    MultipleMatchesFound: Boolean;
    GetMultipleMatches: Boolean;
  end;

  procedure UserOnDebug(Status: PWideChar); stdcall;

var
    Form1: TForm1;
    CDDB: HCDDB;

implementation

{$R *.dfm}

Uses
    MultipleMatches;

procedure UserOnDebug(Status: PWideChar); stdcall;
var
    Response: String;
begin
    Response := Status;

    //* Just display the reply
    Form1.Memo2.Lines.Add(Response);

    //* Detect multiple matches
    if (Copy(Response, 1, 3) = '210')
    AND (NOT Form1.MultipleMatchesFound)
    then begin
        Form1.MultipleMatchesFound := True;
        Form1.GetMultipleMatches := True;
        Exit;
    end;
    if Form1.GetMultipleMatches then begin
        if Response = '.' then begin
            Form1.GetMultipleMatches := False;
            Exit;
        end;
        FormMultipleMatches.ListBox1.Items.Add(Response);
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    //* Load the .dll
    InitCDDBLibrary;

    //* Create an instance
    CDDB := CDDB_Create;

    // check the correct BASS was loaded
	if (HIWORD(BASS_GetVersion) <> BASSVERSION) then
	begin
		MessageBox(0,'An incorrect version of BASS.DLL was loaded',nil,MB_ICONERROR);
		Halt;
	end;

	// Initialize audio - default device, 44100hz, stereo, 16 bits
	if not BASS_Init(-1, 44100, 0, Handle, nil)
        then MessageBox(0,'Error initializing audio!',nil,MB_ICONERROR);

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
    //* Free our object
    CDDB_Free(CDDB);
    //* Call always on program exit
    FreeCDDBLibrary;
    // Free BASS
	BASS_Free;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
    Params: TCDDB_Params;
    Title, Server: String;
    i: Integer;
    MultipleMatchCategory: String;
begin

    //* Clear things
    MultipleMatchesFound := False;
    GetMultipleMatches := False;
    FormMultipleMatches.ListBox1.Clear;
    Memo1.Clear;

    //* Never forget to clear before a query!
    CDDB_SetMultipleMatchCategory(CDDB, '');

    //* Get parameters
    CDDB_GetParams(CDDB, Params);

    //* Set parameters
    Params.Handle := Handle;
    //* Extract the server name from the string
    Server := Copy(ComboBox1.Text, 1, Pos(' ', ComboBox1.Text) - 1);
    Params.ServerAddr := PChar(Server);
    Params.ServerPort := PChar(EditPort.Text);
    //* In milliseconds
    Params.TimeOut := StrToInt(Edit2.Text);
    //* Needed for authentication
    Params.UserName := PChar('3delite');
    Params.AppURL := PChar('www.3delite.hu');
    Params.AppID := PChar('CDDBFreeDBLibrary');
    Params.AppVer := PChar('1.2');
    Params.GetSiteList := False;
    //* Debug stuff
    //* Also useable for custom processing of the reply and also needed for multiple matches processing
    Params.OnDebug := @UserOnDebug;
    //* Set it
    CDDB_SetParams(CDDB, Params);

    //* Make a query
    CDDB_Query(CDDB, PChar(Edit1.Text));

    //* Multiple matches found?
    if MultipleMatchesFound then begin
        if FormMultipleMatches.ShowModal = mrYes then begin
            //* Extract which category match to retrieve
            MultipleMatchCategory := FormMultipleMatches.ListBox1.Items[FormMultipleMatches.ListBox1.ItemIndex];
            MultipleMatchCategory := Copy(MultipleMatchCategory, 1, Pos(' ', MultipleMatchCategory) - 1);
            //* Set which category match to retrieve
            CDDB_SetMultipleMatchCategory(CDDB, PWideChar(MultipleMatchCategory));
            //* Make a query once more with the selected category
            //* As now MultipleMatchCategory is set, the set one will be retrieved
            CDDB_Query(CDDB, PChar(Edit1.Text));
        end else begin
            Exit;
        end;
    end;

    //* Process the reply
    //* Get all the titles
    for i := 0 to CDDB_GetTrackCount(CDDB) - 1 do begin
        Title := CDDB_GetTitle(CDDB, i);
        Memo1.Lines.Add(Title);
    end;

    //* BR
    Memo1.Lines.Add('');

    //* Get additional information
    Memo1.Lines.Add('Genre: ' + CDDB_GetGenre(CDDB));
    Memo1.Lines.Add('Artist: ' + CDDB_GetArtist(CDDB));
    Memo1.Lines.Add('Album: ' + CDDB_GetAlbum(CDDB));
    Memo1.Lines.Add('Year: ' + CDDB_GetYear(CDDB));
    Memo1.Lines.Add('Composer: ' + CDDB_GetComposer(CDDB));
    Memo1.Lines.Add('Copyright: ' + CDDB_GetCopyright(CDDB));
    Memo1.Lines.Add('URL: ' + CDDB_GetURL(CDDB));
    Memo1.Lines.Add('Original Artist: ' + CDDB_GetOriginalArtist(CDDB));
    Memo1.Lines.Add('Revision: ' + IntToStr(CDDB_GetRevision(CDDB)));
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
    //* Get CDDB CDID, first drive
    Edit1.Text := BASS_CD_GetID(0, BASS_CDID_CDDB);
end;

procedure TForm1.Button3Click(Sender: TObject);
var
    Params: TCDDB_SubmitParams;
    ErrorStr: String;
    Genre: String;
    i: Integer;
begin
    //* Get parameters
    CDDB_SubmitGetParams(CDDB, Params);
    //* SMTP server address (your e-mail provider's host name)
    Params.ServerAddr := PChar(Edit14.Text);
    //* Usually 25
    Params.ServerPort := StrToInt(Edit3.Text);
    //* SMTP user name (login name)
    Params.UserName := PChar(Edit4.Text);
    //* SMTP password (password for the login name)
    Params.Password := PChar(Edit5.Text);
    //* User's e-mail address (Success/failure is responded here)
    Params.UserEMail := PChar(Edit6.Text);
    //* Your app's ID
    Params.Agent := PChar('CDDBFreeDBLibrary');
    //* Test or normal submit use CDDB_EMAILADDRESS_SUBMIT for normal submit
    Params.DestEMail := PChar(CDDB_EMAILADDRESS_TESTSUBMIT);
    //* Apply the settings
    CDDB_SubmitSetParams(CDDB, Params);

    //* Clear the list
    CDDB_SubmitClear(CDDB);
    //* Add track titles
    for i := 0 to Memo3.Lines.Count - 1 do begin
        CDDB_SubmitAddTitle(CDDB, PChar(Memo3.Lines[i]));
    end;

    //* Important! Performing a submission requires a category!
    //* Extract the category name from the string
    Genre := Copy(ComboBox3.Text, 1, Pos(' ', ComboBox3.Text) - 1);
    CDDB_SubmitSetGenre(CDDB, PChar(Genre));

    //* Set the additional fields
    CDDB_SubmitSetArtist(CDDB, PChar(Edit7.Text));
    CDDB_SubmitSetAlbum(CDDB, PChar(Edit9.Text));
    CDDB_SubmitSetYear(CDDB, PChar(Edit8.Text));
    CDDB_SubmitSetComposer(CDDB, PChar(Edit10.Text));
    CDDB_SubmitSetCopyright(CDDB, PChar(Edit11.Text));
    CDDB_SubmitSetURL(CDDB, PChar(Edit12.Text));
    CDDB_SubmitSetOrigArtist(CDDB, PChar(Edit13.Text));

    //* Do the submission
    //* Increment Revision to update an existintg entry
    if NOT CDDB_SubmitPerform(CDDB, PChar(Edit1.Text), 0) then begin
        //* Display the error
        Showmessage(CDDB_GetError(CDDB));
    end else begin
        Showmessage('Submission successful.');
    end;
end;

end.
